import * as log from './log'
import * as events from 'events'

export type ChainItemFunc = (message: any) => any

export abstract class AChainItem extends events.EventEmitter {
    protected _label : string
    protected _next : AChainItem | null


    constructor(label : string) {
        super()
        this._label = label
        this._next = null
    }

    public toString() : string {
        return `<${this._label}`
    }

    registerNext(next : AChainItem) :void {
        this._next = next
    }

    send(message : any) : void {
        this.emit('yourturn', message)
    }

    abstract begin() : void
}

export class ChainItem extends AChainItem  {
    private _chainItemFunc : ChainItemFunc

    constructor(label : string) {
        super(label)
    }

    public toString() : string {
        let funcName : string = (this._chainItemFunc.name === "") ? "lambda" : this._chainItemFunc.name
        return `<${funcName}, ${this._label}>`
    }

    registerFunc(func : ChainItemFunc) : void {
        this._chainItemFunc = func
    }

    begin() :void {
        if (this._next != null) {
            this._next.begin()
        }
        this.addListener('yourturn', (message : any) : void => {
            log.info(`Calling ${this.toString()}`)
            let result : any = this._chainItemFunc(message)
            if (this._next == null) {
                log.info(`Last chain item, final result ${result}`)
            } else {
                this._next.send(result)
            }
        })
    }
}

